/* filename:    sunpm.g
** description: The program maximizes the posterior density of the 
**              Clarida-Gali-Gertler Models
** created:     02/06/02
*/

new;
library pgraph, user;
cls;

format /mat /on /mb1 /ros 16,8;


/* Global defaults
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g
#include c:\projects\active\sunspots\gauss1\sunspec.g

/******************************************************** 
** Import data on output, inflation, nominal interest rate
** series (nobs,3)
*/
#include c:\projects\active\sunspots\gauss1\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series[1:nobs,.];   

/******************************************************** 
** Import the information for the prior density
*/
priorfile = "m" $+ lprior $+ "pri.out";
load path = ^priopath prior[npara,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,subT];

/******************************************************** 
** Starting Values for Maximization
*/
psi1  = 2.0;
psi2  = 0.1;
rhor  = 0.4;

pista = 4;
rsta  = 2.0;

kap    = 0.4;
tauinv = 1;

rhog   = 0.7;
rhoa   = 0.9;

r_g_z   = 0.0;
r_eta_r = 0.0;
r_eta_g = 0.0;
r_eta_z = 0.0;

sigr   = 0.3;
sigg   = 0.8;
sigz   = 0.5;
sigeta = 0.1;


para = psi1 | psi2  |rhor | pista | rsta | kap  | tauinv | rhog | rhoa | r_g_z |
       r_eta_r | r_eta_g | r_eta_z | sigr | sigg | sigz | sigeta ;

npara = rows(para);

para  = para.*para_maskinv[.,subT] + para_fix[.,subT].*para_mask[.,subT];

para_names = { PSI1 PSI2 RHOR PSTA RSTA KAP TAUINV RHOG RHOA RGZ RER REG REZ SIGR 
               SIGG SIGZ SIGETA };
save path = ^priopath para_names;

/* Load transformation scheme for parameters
*/
transfile = "trans.out";
load path=^priopath _trspec[npara,4] = ^transfile;
_trspec[.,1] = _trspec[.,1].*para_maskinv[.,subT];

/******************************************************** 
**      Calculate posterior density at starting values
**
*/
cls;
"Prior Density at Starting Value:" priocgg(para,pmean,pstdd,pshape);
lnpy = -objfcn(invtrans(para));
"Posterior at Starting Value:    " lnpy;
"Press key to continue";
w = keyw;
cls;

/********************************************************
**       Maximize the posterior density
**
*/
cc = 0;
x0 = invtrans(para) + cc*rndn(npara,1);

H0= eye(npara)*1E-4;
nit = 1000;
crit= 5E-9;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);
paraest = real( paraest.*para_maskinv[.,subT] + para_fix[.,subT].*para_mask[.,subT] );

oparaest         = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
{lnpy, retcode, obsmean, obsvar} = evalcgg( paraest,mspec,YY);
lnprio  = priocgg( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Model              " mspec;
"Sample             " datasel;
"Parameter Shifts   " SubT;
"===================================";
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);

if lnpy == -1E6 ;
   end;
endif;

/********************************************************
**       Compute Residuals
**
*/

/* Moments */

resid1 = YY - obsmean;
resid1_m1 = meanc(resid1)';
resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

resid2 = (YY - obsmean)./sqrt(obsvar);
resid2_m1 = meanc(resid1)';
resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

/* Autocorrelation */

resid1_ar = zeros(1,3);
resid2_ar = zeros(1,3);

ind = 1;
do until ind > 3;
   resid1_ar[1,ind]=inv(resid1[1:nobs-1,ind]'*resid1[1:nobs-1,ind])
                    *resid1[1:nobs-1,ind]'*resid1[2:nobs,ind];
   resid2_ar[1,ind]=inv(resid2[1:nobs-1,ind]'*resid2[1:nobs-1,ind])
                    *resid2[1:nobs-1,ind]'*resid2[2:nobs,ind];
   ind = ind+1;
endo;


"Raw Residuals";
"-------------";
"Mean:" resid1_m1;
"Stdd:" sqrt(resid1_m2);
"Skew:" resid1_m3./resid1_m2^(3/2);
"Kurt:" resid1_m4./(3*resid1_m2^2);
"Corr:" resid1_ar;
"";
"Standardized Residuals";
"----------------------";
"Mean:" resid2_m1;
"Stdd:" sqrt(resid2_m2);
"Skew:" resid2_m3./resid2_m2^(3/2);
"Kurt:" resid2_m4./(3*resid2_m2^2);
"Corr:" resid2_ar;
"";

titlestr = "Output"  $|
           "Inflation" $|
           "Interest Rate";

ystr     = "Raw Residual " $|
           "Stdd Residual"; 

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
          
begwind;
window(2,3,0);


ser_ind = 1;
do until ser_ind > 3;

   ylabel(ystr[1]);
   title(titlestr[ser_ind]);
   xy(ti,resid1[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

ser_ind = 1;
do until ser_ind > 3;

   ylabel(ystr[2]);
   title(titlestr[ser_ind]);
   xy(ti,resid2[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

endwind;
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpY, lnprio, obsmean, obsvar, modelpara, retcode;

/* likelihood 
*/
modelpara = trans(para);
modelpara = modelpara.*para_maskinv[.,subT] + para_fix[.,subT].*para_mask[.,subT];
 
{lnpY,retcode,obsmean,obsvar} = evalcgg(modelpara,mspec,YY); 

/* Evaluate the Prior distribution
*/
lnprio = priocgg(modelpara, pmean, pstdd, pshape);

/*
locate 25,1;
"Likelihood" lnpY  "Return Code" retcode;
*/

retp(real(-lnpY-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;

#include c:\projects\active\sunspots\gauss1\sunmod.src;
#include c:\projects\active\sunspots\gauss1\sunlh.src;
